/*
 * console.cpp
 *
 *
 */


#include <console.h>
#include <cocoos.h>
#include <inc/usart.h>
#include <inc/utilstypes.h>
#include <inc/parser.h>
#include <inc/conversions.h>

using namespace mcal;


// string queue
static utils::string outputbuf[32];
StrQueue Console::output(outputbuf,32);

static Console* instance;

Console::Console(uint8_t prio, TimeProvider& tmPrvdr):
        _prio(prio), timeProvider(tmPrvdr) {
    instance = this;
}

Console::~Console() {}

void Console::create() {
	task_create( task, _prio, 0, 0, 0 );
}

void Console::task() {
	static utils::string str;
	static mcal::usart::ptr u = mcal::usart::instance(0);
    static mcal::rawwriter terminal(*u);
	task_open();

	for (;;) {
	    // check the output queue every ms
		task_wait(1);

		if ( output.pop(&str) ) {
			// we have an output string to print
			terminal.write((uint8_t*)str.get(), str.len());
		}
	}

	task_close();
}

void Console::printTimestampedString(const char *str) {
    // create a string to write, consisting of a timestamp+string to write
    char timebuf[32];
    uint32ToStringBase10(timebuf, timeProvider.getTime());
    utils::string out(timebuf);
    out.append(": ");
    out.append(str);

    // push the string to the output queue
    output.push(out);
}


extern "C" {
void printString(const char *str) {
    instance->printTimestampedString(str);
}

}
